# ticktracer
`ticktracer` is a simple Python application that allows you to trace your latency to crypto exchanges, and was developed to analyze the shortest latency for exchanges hosted on Alibaba Cloud. Because the machine running `ticktracer` and the target exchange leverage the same NTP Provider, we can safely assume that the any clock skew between the two providers is negligible. You can verify the time synchronization status of your current machine through the command `chronyc sources`.

Currently only websocket data feeds are supported. It works by establishing a websocket connection to a crypto exchange api endpoint, decrypting the session with Wireshark under the hood, and comparing the timestamp of packets with those provided by the exchange. Packet timestamps are received by `libpcap` from the kernel and are not generated by `ticktracer` itself, unlike other open source projects. See [here](https://wiki.wireshark.org/Timestamps) for more information.

![Architecture overview](architecture.png)
### Supported Exchanges
- Binance
- Bybit
- OKX

## Requirements
Ticktracer has been tested on the following operating systems:
- macOS >= 14.0
- Debian >= 12.0
- Ubuntu >= 22.0

In addition, the following external dependencies are required:
- `wireshark` or `tshark` >= 4.0.7
- `libssh-dev` for Debian-based systems or `openssl` for MacOS

## Installation
### MacOS
Install with [Homebrew](https://brew.sh/).
```
brew install wireshark openssl
python3 -m pip install ticktracer
```
### Linux
Install with apt.
```
apt update && apt install wireshark libssh-dev
python3 -m pip install ticktracer
```

## Usage
```
usage: ticktracer [-h] [-e EXCHANGE}] -p PAIR -d DURATION [-i INTERFACE]

options:
  -h, --help            show this help message and exit
  -e, --exchange_id {okx,binance,bybit}
                        The exchange to connect with.
  -p, --pair PAIR       The trading pair to monitor. Look up the specific format on the exchange's API documentation
  -d, --duration DURATION
                        Duration in seconds to monitor.
  -i, --interface INTERFACE
                        The network interface to listen on. Default is eth0 for Linux or en0 for macOS.
```
After the monitoring duration has elapsed, the script will output the average latency in milliseconds:
```
Total samples taken: 11567
Latency report:
+------------+-------------------------+
| Percentile | Latency in milliseconds |
+------------+-------------------------+
|    MIN     |          3.104          |
|    25%     |          9.151          |
|    50%     |          10.401         |
|    75%     |          11.762         |
|    90%     |          13.08          |
|    95%     |          14.102         |
|    99%     |          19.474         |
|   99.9%    |          32.975         |
|    MAX     |          48.023         |
+------------+-------------------------+
```